/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.jenkins.results.parser;

import java.io.File;
import java.io.IOException;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/**
 * @author Michael Hashimoto
 */
public class JobFactory {

	public static Job newJob(String jobName) {
		return newJob(jobName, null, null);
	}

	public static Job newJob(String jobName, String testSuiteName) {
		return newJob(jobName, testSuiteName, null);
	}

	public static Job newJob(
		String jobName, String testSuiteName, String portalBranchName) {

		Job job = _jobs.get(jobName);

		if (job != null) {
			return job;
		}

		if (jobName.contains("test-plugins-acceptance-pullrequest(")) {
			PluginsRepositoryJob pluginsRepositoryJob =
				new PluginsRepositoryJob(jobName);

			_jobs.put(jobName, pluginsRepositoryJob);

			return pluginsRepositoryJob;
		}

		if (jobName.contains("test-portal-acceptance-pullrequest(")) {
			PortalAcceptancePullRequestJob portalAcceptancePullRequestJob =
				new PortalAcceptancePullRequestJob(jobName, testSuiteName);

			GitWorkingDirectory gitWorkingDirectory =
				portalAcceptancePullRequestJob.getGitWorkingDirectory();

			String subrepositoryModuleName = _getSubrepositoryModuleName(
				gitWorkingDirectory);

			RepositoryJob repositoryJob = null;

			if (subrepositoryModuleName == null) {
				repositoryJob = portalAcceptancePullRequestJob;
			}
			else {
				repositoryJob = new SubrepositoryAcceptancePullRequestJob(
					jobName);

				Properties buildProperties = null;

				try {
					buildProperties =
						JenkinsResultsParserUtil.getBuildProperties();
				}
				catch (IOException ioe) {
					throw new RuntimeException(
						"Unable to get build properties", ioe);
				}

				repositoryJob.setRepositoryDir(
					new File(
						JenkinsResultsParserUtil.combine(
							buildProperties.getProperty("base.repository.dir"),
							"/", subrepositoryModuleName)));
			}

			_jobs.put(jobName, repositoryJob);

			return repositoryJob;
		}

		if (jobName.contains("test-portal-acceptance-upstream(")) {
			_jobs.put(jobName, new PortalAcceptanceUpstreamJob(jobName));

			return _jobs.get(jobName);
		}

		if (jobName.equals("test-portal-release")) {
			_jobs.put(jobName, new PortalReleaseJob(jobName, portalBranchName));

			return _jobs.get(jobName);
		}

		if (jobName.contains("test-portal-upstream(")) {
			_jobs.put(jobName, new PortalUpstreamJob(jobName));

			return _jobs.get(jobName);
		}

		if (jobName.contains("test-subrepository-acceptance-pullrequest(")) {
			_jobs.put(
				jobName, new SubrepositoryAcceptancePullRequestJob(jobName));

			return _jobs.get(jobName);
		}

		throw new IllegalArgumentException("Invalid job name " + jobName);
	}

	private static String _getSubrepositoryModuleName(
		GitWorkingDirectory gitWorkingDirectory) {

		List<File> currentBranchModifiedFiles =
			gitWorkingDirectory.getModifiedFilesList();

		if (currentBranchModifiedFiles.size() == 1) {
			File modifiedFile = currentBranchModifiedFiles.get(0);

			String modifiedFileName = modifiedFile.getName();

			if (modifiedFileName.equals("ci-merge")) {
				File moduleDir = modifiedFile.getParentFile();

				List<File> lfrBuildPortalFiles =
					JenkinsResultsParserUtil.findFiles(
						moduleDir, "\\.lfrbuild-portal");

				if (lfrBuildPortalFiles.isEmpty()) {
					File gitrepoFile = new File(moduleDir, ".gitrepo");

					Properties properties =
						JenkinsResultsParserUtil.getProperties(gitrepoFile);

					String subrepositoryRemote = properties.getProperty(
						"remote");

					return subrepositoryRemote.replaceAll(
						".*(com-liferay-[^\\.]+)\\.git", "$1");
				}
			}
		}

		return null;
	}

	private static final Map<String, Job> _jobs = new HashMap<>();

}